package nl.amis.workshopse;

import java.sql.Types;
import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.jdbc.core.SqlOutParameter;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.jdbc.object.StoredProcedure;
/**
 *  Klasse bevat de business rules van een applicatie
 *  Deze business rules zijn gebaseerd op elders (database) beschikbare business rules 
 *
 *  @author     Aino Andriessen
 */
public class BusinessRules2 
{
    
        /** The datasource to connect to the database   */
    private DriverManagerDataSource ds;
        /**
         *  Calculation1 - use the database calculation
         *  (pInput * pInput) / 2
         *  @param      pInput      De input waarde
         *  @return                 De berekende waarde
         */
    public double calc1 (double pInput)
    {
        WeirdCalculation1SP sproc = new WeirdCalculation1SP(ds);
        return sproc.execute (pInput);
    }

        
    private class WeirdCalculation1SP extends StoredProcedure {
            /** The stored procedure that is associated with this StoredProcedure   */
        public static final String SQL = "weirdcalculations.weirdcalculation1";

            /**
             *  Constructor, initialize the StoredProcedure
             *  @param  ds  The datasource that is to be used for this stored procedure
             */
        public WeirdCalculation1SP(DataSource ds) {
            setDataSource(ds);
            setFunction(true);
            setSql(SQL);
            declareParameter(new SqlOutParameter("result", Types.DOUBLE));
            declareParameter(new SqlParameter ("pinput", Types.DOUBLE));
            compile();
        }

            /**
             *  Execute the stored procedure
             *
             *  @param      pInput      The value to perform the calculation with
             *  @return                 The calculated value
             */
        public double execute(double pInput) 
        {
            Map input = new HashMap();
            input.put ("pinput",new Double (pInput));
            Map out = execute(input);
            if (!out.isEmpty())
            {
                Double d = (Double)out.get("result");
                return d.doubleValue();
            }
            return -1.0;

        }
    }

        /**
         *  Just a normal getter
         *  @return     The datasource
         */
    public DriverManagerDataSource getDs()
    {
        return ds;
    }

        /**
         *  Just a normal setter. It is used for runtime injection
         *  @param  ds  The datasource to be set.
         */
    public void setDs(DriverManagerDataSource ds)
    {
        this.ds = ds;
    }

}